@@ -20,7 +20,6 @@ from numpy.linalg import LinAlgError
 
 from scipy.optimize import leastsq as scipy_leastsq
 from scipy.optimize import fmin as scipy_fmin
-from scipy.optimize import anneal as scipy_anneal
 from scipy.optimize.lbfgsb import fmin_l_bfgs_b as scipy_lbfgsb
 
 # check for scipy.optimize.minimize
@@ -100,6 +99,18 @@ def check_ast_errors(error):
             return msg
     return None
 
+SCALAR_METHODS = {'nelder': 'Nelder-Mead',
+                  'powell': 'Powell',
+                  'cg': 'CG',
+                  'bfgs': 'BFGS',
+                  'newton': 'Newton-CG',
+                  'lbfgs': 'L-BFGS-B',
+                  'l-bfgs':'L-BFGS-B',
+                  'tnc': 'TNC',
+                  'cobyla': 'COBYLA',
+                  'slsqp': 'SLSQP',
+                  'dogleg': 'dogleg',
+                  'trust-ncg': 'trust-ncg'}
 
 class Minimizer(object):
     """general minimizer"""
@@ -124,8 +135,21 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
         self.scale_covar = scale_covar
         self.nfev = 0
         self.nfree = 0
+        self.ndata = 0
+        self.nvarys = 0
+        self.ier = 0
+        self.success = None
         self.message = None
+        self.lmdif_message = None
+        self.chisqr = None
+        self.redchi = None
+        self.covar = None
+        self.errorbars = None
+        self.residual = None
         self.var_map = []
+        self.vars = []
+        self.params = []
+        self.updated = []
         self.jacfcn = None
         self.asteval = Interpreter()
         self.namefinder = NameFinder()
@@ -245,7 +269,6 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
         self.nfev = 0
         self.var_map = []
         self.vars = []
-        self.vmin, self.vmax = [], []
         for name, par in self.params.items():
             if par.expr is not None:
                 par.ast = self.asteval.parse(par.expr)
@@ -261,9 +284,6 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
             elif par.vary:
                 self.var_map.append(name)
                 self.vars.append(par.setup_bounds())
-                # self.vars.append(par.set_internal_value())
-                #self.vmin.append(par.min)
-                #self.vmax.append(par.max)
 
             self.asteval.symtable[name] = par.value
             par.init_value = par.value
@@ -290,26 +310,6 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
             if hasattr(par, 'ast'):
                 delattr(par, 'ast')
 
-    def anneal(self, schedule='cauchy', **kws):
-        """
-        use simulated annealing
-        """
-        sched = 'fast'
-        if schedule in ('cauchy', 'boltzmann'):
-            sched = schedule
-
-        self.prepare_fit()
-        sakws = dict(full_output=1, schedule=sched,
-                     maxiter=2000 * (self.nvarys + 1))
-
-        sakws.update(self.kws)
-        sakws.update(kws)
-        print("WARNING:  scipy anneal appears unusable!")
-        saout = scipy_anneal(self.penalty, self.vars, **sakws)
-        self.sa_out = saout
-        self.unprepare_fit()
-        return
-
     def lbfgsb(self, **kws):
         """
         use l-bfgs-b minimization
@@ -317,7 +317,6 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
         self.prepare_fit()
         lb_kws = dict(factr=1000.0, approx_grad=True, m=20,
                       maxfun=2000 * (self.nvarys + 1),
-                      # bounds = zip(self.vmin, self.vmax),
                       )
         lb_kws.update(self.kws)
         lb_kws.update(kws)
@@ -347,7 +346,7 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
 
         fmin_kws.update(kws)
         ret = scipy_fmin(self.penalty, self.vars, **fmin_kws)
-        xout, fout, iter, funccalls, warnflag, allvecs = ret
+        xout, fout, niter, funccalls, warnflag, allvecs = ret
         self.nfev = funccalls
         self.chisqr = self.residual = self.__residual(xout)
         self.ndata = 1
@@ -368,7 +367,6 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
           CG  (conjugate gradient)
           BFGS
           Newton-CG
-          Anneal
           L-BFGS-B
           TNC
           COBYLA
@@ -512,6 +510,9 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
             par.stderr, par.correl = 0, None
             has_expr = has_expr or par.expr is not None
 
+        # self.errorbars = error bars were successfully estimated
+        self.errorbars = self.covar is not None
+
         if self.covar is not None:
             if self.scale_covar:
                 self.covar = self.covar * sum_sqr / self.nfree
@@ -520,10 +521,14 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
                 par = self.params[varname]
                 par.stderr = sqrt(self.covar[ivar, ivar])
                 par.correl = {}
-                for jvar, varn2 in enumerate(self.var_map):
-                    if jvar != ivar:
-                        par.correl[varn2] = (self.covar[ivar, jvar] /
-                             (par.stderr * sqrt(self.covar[jvar, jvar])))
+                try:
+                    self.errorbars = self.errorbars and (par.stderr > 0.0)
+                    for jvar, varn2 in enumerate(self.var_map):
+                        if jvar != ivar:
+                            par.correl[varn2] = (self.covar[ivar, jvar] /
+                                 (par.stderr * sqrt(self.covar[jvar, jvar])))
+                except:
+                    self.errorbars = False
 
             uvars = None
             if has_expr:
@@ -538,72 +543,50 @@ or set  leastsq_kws['maxfev']  to increase this maximum."""
                     uvars = None
 
                 if uvars is not None:
-                    for pname, par in self.params.items():
+                    for par in self.params.values():
                         eval_stderr(par, uvars, self.var_map,
                                     self.params, self.asteval)
                     # restore nominal values
                     for v, nam in zip(uvars, self.var_map):
                         self.asteval.symtable[nam] = v.nominal_value
 
-        self.errorbars = True
-        if self.covar is None:
-            self.errorbars = False
-            self.message = '%s. Could not estimate error-bars'
+        if not self.errorbars:
+            self.message = '%s. Could not estimate error-bars'  % self.message
 
         np.seterr(**orig_warn_settings)
         self.unprepare_fit()
         return self.success
 
+    def minimize(self, method='leastsq'):
+        """perform minimization using supplied method"""
+        function = self.leastsq
+        kwargs = {}
+
+        user_method = method.lower()
+        if user_method.startswith('least'):
+            function = self.leastsq
+        elif HAS_SCALAR_MIN:
+            function = self.scalar_minimize
+            for key, val in SCALAR_METHODS.items():
+                if (key.lower().startswith(user_method) or
+                    val.lower().startswith(user_method)):
+                    kwargs = dict(method=val)
+        elif (user_method.startswith('nelder') or
+              user_method.startswith('fmin')):
+            function = self.fmin
+        elif user_method.startswith('lbfgsb'):
+            function = self.lbfgsb
+
+        return function(**kwargs)
+
 
 def minimize(fcn, params, method='leastsq', args=None, kws=None,
-             scale_covar=True, engine=None, iter_cb=None, **fit_kws):
+             scale_covar=True, iter_cb=None, **fit_kws):
     """simple minimization function,
     finding the values for the params which give the
     minimal sum-of-squares of the array return by fcn
     """
     fitter = Minimizer(fcn, params, fcn_args=args, fcn_kws=kws,
                        iter_cb=iter_cb, scale_covar=scale_covar, **fit_kws)
-
-    _scalar_methods = {'nelder': 'Nelder-Mead',
-                       'powell': 'Powell',
-                       'cg': 'CG',
-                       'bfgs': 'BFGS',
-                       'newton': 'Newton-CG',
-                       'anneal': 'Anneal',
-                       'lbfgs': 'L-BFGS-B',
-                       'l-bfgs':'L-BFGS-B',
-                       'tnc': 'TNC',
-                       'cobyla': 'COBYLA',
-                       'slsqp': 'SLSQP',
-                       'dogleg': 'dogleg',
-                       'trust-ncg': 'trust-ncg'}
-
-    _fitmethods = {'anneal': 'anneal',
-                   'nelder': 'fmin',
-                   'lbfgsb': 'lbfgsb',
-                   'leastsq': 'leastsq'}
-
-    if engine is not None:
-        method = engine
-    meth = method.lower()
-
-    fitfunction = None
-    kwargs = {}
-    # default and most common option: use leastsq method.
-    if meth == 'leastsq':
-        fitfunction = fitter.leastsq
-    else:
-        # if scalar_minimize() is supported and method is in list, use it.
-        if HAS_SCALAR_MIN:
-            for name, method in _scalar_methods.items():
-                if meth.startswith(name):
-                    fitfunction = fitter.scalar_minimize
-                    kwargs = dict(method=method)
-        # look for other built-in methods
-        if fitfunction is None:
-            for name, method in _fitmethods.items():
-                if meth.startswith(name):
-                    fitfunction = getattr(fitter, method)
-    if fitfunction is not None:
-        fitfunction(**kwargs)
+    fitter.minimize(method=method)
     return fitter
