@@ -24,7 +24,7 @@ class Parameters(OrderedDict):
     add_many()
     """
     def __init__(self, *args, **kwds):
-        OrderedDict.__init__(self)
+        super(Parameters, self).__init__(self)
         self.update(*args, **kwds)
 
     def __setitem__(self, key, value):
@@ -36,6 +36,13 @@ class Parameters(OrderedDict):
         OrderedDict.__setitem__(self, key, value)
         value.name = key
 
+    def __add__(self, other):
+        "add Parameters objects"
+        if not isinstance(other, Parameters):
+            raise ValueError("'%s' is not a Parameters object" % other)
+        self.update(other)
+        return self
+
     def add(self, name, value=None, vary=True, min=None, max=None, expr=None):
         """convenience function for adding a Parameter:
         with   p = Parameters()
@@ -62,6 +69,14 @@ class Parameters(OrderedDict):
         for para in parlist:
             self.add(*para)
 
+    def valuesdict(self):
+        """return on ordered dictionary of name:value pairs for each Parameter.
+        This is distinct from the Parameters itself, as it has values of
+        the Parameeter values, not the full Parameter object """
+
+        return OrderedDict(((p.name, p.value) for p in self.values()))
+
+
 class Parameter(object):
     """A Parameter is the basic Parameter going
     into Fit Model.  The Parameter holds many attributes:
@@ -77,12 +92,26 @@ class Parameter(object):
         self.min = min
         self.max = max
         self.vary = vary
-        self.expr = expr
+        self._expr = expr
         self.deps   = None
         self.stderr = None
         self.correl = None
+        self.from_internal = lambda val: val
         self._init_bounds()
 
+    def set(self, value=None, vary=None, min=None, max=None, expr=None):
+        "set value, vary, min, max, expr with keyword args"
+        if value is not None:
+            self._val = value
+        if vary is not None:
+            self.vary = vary
+        if min is not None:
+            self.min = min
+        if max is not None:
+            self.max = max
+        if expr is not None:
+            self.expr = expr
+
     def _init_bounds(self):
         """make sure initial bounds are self-consistent"""
         #_val is None means - infinity.
@@ -93,6 +122,8 @@ class Parameter(object):
                 self._val = self.min
         elif self.min is not None:
             self._val = self.min
+        elif self.max is not None:
+            self._val = self.max
         self.setup_bounds()
 
     def __getstate__(self):
@@ -171,7 +202,10 @@ class Parameter(object):
         """get value, with bounds applied"""
         if (self._val is not nan and
             isinstance(self._val, uncertainties.Variable)):
-            self._val = self._val.nominal_value
+            try:
+                self._val = self._val.nominal_value
+            except AttributeError:
+                pass
 
         if self.min is None:
             self.min = -inf
@@ -198,6 +232,19 @@ class Parameter(object):
     def value(self, val):
         "set value"
         self._val = val
+
+    @property
+    def expr(self):
+        "get expression"
+        return self._expr
+
+    @expr.setter
+    def expr(self, val):
+        "set expr"
+        if val == '':
+            val = None
+        self._expr = val
+
     def __str__(self):
         "string"
         return self.__repr__()
